@@ -6,12 +6,13 @@ representation such as JsonSchema
 __all__ = [
     "csvgen",
     "dotgen",
+    "docgen",
     "golrgen",
     "graphqlgen",
+    "javagen",
     "jsonldcontextgen",
     "jsonldgen",
     "jsonschemagen",
-    "lpgen",
     "markdowngen",
     "namespacegen",
     "owlgen",
