@@ -1,15 +1,24 @@
-from dataclasses import dataclass, field
-from typing import Dict, Set, List, Union, cast
 import logging
+from dataclasses import dataclass, field
+from typing import Dict, List, Set, Union, cast
 
+from linkml_runtime.linkml_model.meta import (ClassDefinition,
+                                              ClassDefinitionName, Definition,
+                                              DefinitionName, Element,
+                                              ElementName, EnumDefinition,
+                                              EnumDefinitionName,
+                                              SchemaDefinition, SlotDefinition,
+                                              SlotDefinitionName,
+                                              SubsetDefinitionName,
+                                              TypeDefinition,
+                                              TypeDefinitionName)
+from linkml_runtime.utils.metamodelcore import empty_dict
+from linkml_runtime.utils.yamlutils import TypedNode
 from rdflib import URIRef
 
-from linkml_runtime.linkml_model.meta import SchemaDefinition, Element, Definition, ClassDefinition, SlotDefinitionName, \
-    ClassDefinitionName, TypeDefinitionName, DefinitionName, SlotDefinition, ElementName, TypeDefinition, \
-    SubsetDefinitionName, EnumDefinitionName, EnumDefinition
-from linkml_runtime.utils.metamodelcore import empty_dict
-from linkml.utils.typereferences import RefType, ClassType, TypeType, SlotType, References, SubsetType, EnumType
-from linkml_runtime.utils.yamlutils import TypedNode
+from linkml.utils.typereferences import (ClassType, EnumType, References,
+                                         RefType, SlotType, SubsetType,
+                                         TypeType)
 
 
 def empty_references() -> field:
@@ -24,45 +33,81 @@ class SchemaSynopsis:
     schema: SchemaDefinition = field(repr=False, compare=False)
 
     # References by type -- set by add_ref
-    typerefs: Dict[TypeDefinitionName, References] = empty_dict()       # Type name to all references
-    slotrefs: Dict[SlotDefinitionName, References] = empty_dict()       # Slot name to all references
-    classrefs: Dict[ClassDefinitionName, References] = empty_dict()     # Class name to all references
-    subsetrefs: Dict[SubsetDefinitionName, References] = empty_dict()   # Subset name to references
-    enumrefs: Dict[EnumDefinitionName, References] = empty_dict()       # Enum name to references
+    typerefs: Dict[
+        TypeDefinitionName, References
+    ] = empty_dict()  # Type name to all references
+    slotrefs: Dict[
+        SlotDefinitionName, References
+    ] = empty_dict()  # Slot name to all references
+    classrefs: Dict[
+        ClassDefinitionName, References
+    ] = empty_dict()  # Class name to all references
+    subsetrefs: Dict[
+        SubsetDefinitionName, References
+    ] = empty_dict()  # Subset name to references
+    enumrefs: Dict[
+        EnumDefinitionName, References
+    ] = empty_dict()  # Enum name to references
 
     # Type specific
-    typebases: Dict[str, Set[TypeDefinitionName]] = empty_dict()          # Base referencing types (direct and indirect)
-    typeofs: Dict[TypeDefinitionName, TypeDefinitionName] = empty_dict()  # Type to specializations
+    typebases: Dict[
+        str, Set[TypeDefinitionName]
+    ] = empty_dict()  # Base referencing types (direct and indirect)
+    typeofs: Dict[
+        TypeDefinitionName, TypeDefinitionName
+    ] = empty_dict()  # Type to specializations
 
     # Slot specific
-    slotclasses: Dict[SlotDefinitionName, Set[ClassDefinitionName]] = empty_dict()    # Slot to including classes
-    definingslots: Dict[SlotDefinitionName, Set[ClassDefinitionName]] = empty_dict()  # Slot to defining decls
-    slotusages: Dict[SlotDefinitionName, Set[ClassDefinitionName]] = empty_dict()     # Slot to overriding classes
-    owners: Dict[SlotDefinitionName, Set[ClassDefinitionName]] = empty_dict()         # Slot to owning classes (sb. 1)
-    inverses: Dict[str, Set[str]] = empty_dict()                        # Slots declared as inverses of other slots
+    slotclasses: Dict[
+        SlotDefinitionName, Set[ClassDefinitionName]
+    ] = empty_dict()  # Slot to including classes
+    definingslots: Dict[
+        SlotDefinitionName, Set[ClassDefinitionName]
+    ] = empty_dict()  # Slot to defining decls
+    slotusages: Dict[
+        SlotDefinitionName, Set[ClassDefinitionName]
+    ] = empty_dict()  # Slot to overriding classes
+    owners: Dict[
+        SlotDefinitionName, Set[ClassDefinitionName]
+    ] = empty_dict()  # Slot to owning classes (sb. 1)
+    inverses: Dict[
+        str, Set[str]
+    ] = empty_dict()  # Slots declared as inverses of other slots
 
     # Class specific
-    ownslots: Dict[ClassDefinitionName, Set[SlotDefinitionName]] = empty_dict() # Slots directly owned by class
+    ownslots: Dict[
+        ClassDefinitionName, Set[SlotDefinitionName]
+    ] = empty_dict()  # Slots directly owned by class
 
     # Enum specific
-    codesets: Dict[URIRef, Set[EnumDefinitionName]] = empty_dict()  # Code set URI to enumeration definition
+    codesets: Dict[
+        URIRef, Set[EnumDefinitionName]
+    ] = empty_dict()  # Code set URI to enumeration definition
 
     # Class to slot domains == class.slots
 
     # Slot or Class (Definition) specific
-    roots: References = empty_references()                      # Definitions with no parents
-    isarefs: Dict[DefinitionName, References] = empty_dict()    # Definition to isa references
-    mixinrefs: Dict[DefinitionName, References] = empty_dict()  # Mixin to referencing classes or slots
-    mixins: References = empty_references()                     # Definitions declared as mixin
-    abstracts: References = empty_references()                  # Definitions declared as abstract
-    applytos: References = empty_references()                   # Definitions that include applytos
-    applytorefs: Dict[DefinitionName, References] = empty_dict()  # Definition to applyier
+    roots: References = empty_references()  # Definitions with no parents
+    isarefs: Dict[
+        DefinitionName, References
+    ] = empty_dict()  # Definition to isa references
+    mixinrefs: Dict[
+        DefinitionName, References
+    ] = empty_dict()  # Mixin to referencing classes or slots
+    mixins: References = empty_references()  # Definitions declared as mixin
+    abstracts: References = empty_references()  # Definitions declared as abstract
+    applytos: References = empty_references()  # Definitions that include applytos
+    applytorefs: Dict[
+        DefinitionName, References
+    ] = empty_dict()  # Definition to applyier
 
     # Slot or Type specific
-    rangerefs: Dict[ElementName, Set[SlotDefinitionName]] = empty_dict()  # Type or class to range slot
+    rangerefs: Dict[
+        ElementName, Set[SlotDefinitionName]
+    ] = empty_dict()  # Type or class to range slot
 
     # Element - any type
-    inschema: Dict[str, References] = empty_references()        # Schema name to elements
+    inschema: Dict[str, References] = empty_references()  # Schema name to elements
 
     def __post_init__(self):
         for k, v in self.schema.slots.items():
@@ -86,8 +131,9 @@ class SchemaSynopsis:
             for slotname in owned_slots:
                 self.owners.setdefault(slotname, set()).add(cls.name)
 
-
-    def summarize_slot_definition(self, k: SlotDefinitionName, v: SlotDefinition) -> None:
+    def summarize_slot_definition(
+        self, k: SlotDefinitionName, v: SlotDefinition
+    ) -> None:
         """
         Summarize a slot definition
         :param k: slot name
@@ -98,9 +144,18 @@ class SchemaSynopsis:
         if v.domain:
             self.add_ref(SlotType, k, ClassType, v.domain)
         self.rangerefs.setdefault(v.range, set()).add(k)
-        self.add_ref(SlotType, k, ClassType if v.range in self.schema.classes else
-                                  EnumType if v.range in self.schema.enums else
-                                  TypeType if v.range in self.schema.types else None, v.range)
+        self.add_ref(
+            SlotType,
+            k,
+            ClassType
+            if v.range in self.schema.classes
+            else EnumType
+            if v.range in self.schema.enums
+            else TypeType
+            if v.range in self.schema.types
+            else None,
+            v.range,
+        )
 
     def summarize_type_definition(self, k: TypeDefinitionName, v: TypeDefinition):
         """
@@ -117,7 +172,9 @@ class SchemaSynopsis:
         if v.base:
             self.typebases.setdefault(v.base, set()).add(k)
 
-    def summarize_class_definition(self, k: ClassDefinitionName, v: ClassDefinition) -> None:
+    def summarize_class_definition(
+        self, k: ClassDefinitionName, v: ClassDefinition
+    ) -> None:
         """
         Summarize class definition element
 
@@ -146,8 +203,9 @@ class SchemaSynopsis:
         """
         self.summarize_element(EnumType, k, v)
 
-
-    def summarize_definition(self, typ: RefType, k: DefinitionName, v: Definition) -> None:
+    def summarize_definition(
+        self, typ: RefType, k: DefinitionName, v: Definition
+    ) -> None:
         """
         Summarize slot and class definitions
 
@@ -185,12 +243,20 @@ class SchemaSynopsis:
         :return:
         """
         if k != v.name:
-            raise ValueError("{typ} name mismatch: {k} != {v.name}")        # should never happen
+            raise ValueError(
+                "{typ} name mismatch: {k} != {v.name}"
+            )  # should never happen
         for subset in v.in_subset:
             self.add_ref(typ, k, SubsetType, subset)
 
-    def add_ref(self, fromtype: RefType, fromname: ElementName, totype: RefType, toname: ElementName, ) -> None:
-        """ Add an inverse reference, indicating that to type/name is referenced by from type/name
+    def add_ref(
+        self,
+        fromtype: RefType,
+        fromname: ElementName,
+        totype: RefType,
+        toname: ElementName,
+    ) -> None:
+        """Add an inverse reference, indicating that to type/name is referenced by from type/name
 
         :param fromtype: Referencer type
         :param fromname: Referencer name
@@ -199,56 +265,82 @@ class SchemaSynopsis:
         :return:
         """
         if totype is ClassType:
-            self.classrefs.setdefault(ClassDefinitionName(toname), References()).addref(fromtype, fromname)
+            self.classrefs.setdefault(ClassDefinitionName(toname), References()).addref(
+                fromtype, fromname
+            )
         elif totype is SlotType:
-            self.slotrefs.setdefault(SlotDefinitionName(toname), References()).addref(fromtype, fromname)
+            self.slotrefs.setdefault(SlotDefinitionName(toname), References()).addref(
+                fromtype, fromname
+            )
         elif totype is TypeType:
-            self.typerefs.setdefault(TypeDefinitionName(toname), References()).addref(fromtype, fromname)
+            self.typerefs.setdefault(TypeDefinitionName(toname), References()).addref(
+                fromtype, fromname
+            )
         elif totype is SubsetType:
-            self.subsetrefs.setdefault(SubsetDefinitionName(toname), References()).addref(fromtype, fromname)
+            self.subsetrefs.setdefault(
+                SubsetDefinitionName(toname), References()
+            ).addref(fromtype, fromname)
         elif totype is EnumType:
-            self.enumrefs.setdefault(SlotDefinitionName(toname), References()).addref(fromtype, fromname)
+            self.enumrefs.setdefault(SlotDefinitionName(toname), References()).addref(
+                fromtype, fromname
+            )
         else:
             raise TypeError("Unknown typ: {typ}")
 
     def _ancestor_is_owned(self, slot: SlotDefinition) -> bool:
-        return bool(slot.is_a) and (slot.is_a in self.owners or self._ancestor_is_owned(self.schema.slots[slot.is_a]))
+        return bool(slot.is_a) and (
+            slot.is_a in self.owners
+            or self._ancestor_is_owned(self.schema.slots[slot.is_a])
+        )
 
     def errors(self) -> List[str]:
         def format_undefineds(refs: Set[Union[str, TypedNode]]) -> List[str]:
-            return [f'{TypedNode.yaml_loc(ref)}: {ref}' for ref in refs]
+            return [f"{TypedNode.yaml_loc(ref)}: {ref}" for ref in refs]
 
         rval = []
         undefined_classes = set(self.classrefs.keys()) - set(self.schema.classes.keys())
         if undefined_classes:
-            rval += [f"\tUndefined class references: "
-                     f"{', '.join(format_undefineds(undefined_classes))}"]
+            rval += [
+                f"\tUndefined class references: "
+                f"{', '.join(format_undefineds(undefined_classes))}"
+            ]
         undefined_slots = set(self.slotrefs.keys()) - set(self.schema.slots.keys())
         if undefined_slots:
-            rval += [f"\tUndefined slot references: "
-                     f"{', '.join(format_undefineds(undefined_slots))}"]
+            rval += [
+                f"\tUndefined slot references: "
+                f"{', '.join(format_undefineds(undefined_slots))}"
+            ]
         undefined_types = set(self.typerefs.keys()) - set(self.schema.types.keys())
         if undefined_types:
-            rval += [f"\tUndefined type references: "
-                     f"{', '.join(format_undefineds(undefined_types))}"]
-        undefined_subsets = set(self.subsetrefs.keys()) - set(self.schema.subsets.keys())
+            rval += [
+                f"\tUndefined type references: "
+                f"{', '.join(format_undefineds(undefined_types))}"
+            ]
+        undefined_subsets = set(self.subsetrefs.keys()) - set(
+            self.schema.subsets.keys()
+        )
         if undefined_subsets:
-            rval += [f"\tUndefined subset references: "
-                     f"{', '.join(format_undefineds(undefined_subsets))}"]
+            rval += [
+                f"\tUndefined subset references: "
+                f"{', '.join(format_undefineds(undefined_subsets))}"
+            ]
         undefined_enums = set(self.enumrefs.keys()) - set(self.schema.enums.keys())
         if undefined_enums:
-            rval += [f"\tUndefined enun references: "
-                     f"{', '.join(format_undefineds(undefined_enums))}"]
+            rval += [
+                f"\tUndefined enun references: "
+                f"{', '.join(format_undefineds(undefined_enums))}"
+            ]
 
         # Inlined slots must be multivalued (not a inviolable rule, but we make assumptions about this elsewhere in
         # the python generator
         for slot in self.schema.slots.values():
             if slot.inlined and not slot.multivalued and slot.identifier:
-                rval += [f'\t{TypedNode.yaml_loc(slot.name)} Slot {slot.name} is declared inline but single valued']
+                rval += [
+                    f"\t{TypedNode.yaml_loc(slot.name)} Slot {slot.name} is declared inline but single valued"
+                ]
         return rval
 
     def summary(self) -> str:
-
         def summarize_refs(refs: Dict[ElementName, References]) -> str:
             clsrefs, slotrefs, typerefs, enumrefs = set(), set(), set(), set()
             if refs is not None:
@@ -257,17 +349,19 @@ class SchemaSynopsis:
                     slotrefs.update(cr.slotrefs)
                     typerefs.update(cr.typerefs)
                     enumrefs.update(cr.enumrefs)
-            return f"\tReferenced by: {len(clsrefs)} classes, {len(slotrefs)} slots, " \
-                   f"{len(typerefs)} types, {len(enumrefs)} enums "
+            return (
+                f"\tReferenced by: {len(clsrefs)} classes, {len(slotrefs)} slots, "
+                f"{len(typerefs)} types, {len(enumrefs)} enums "
+            )
 
-        def recurse_types(typ: TypeDefinitionName, indent: str='\t\t\t') -> List[str]:
-            rval = [f"{indent}{typ}" + (':' if typ in self.typeofs else '')]
+        def recurse_types(typ: TypeDefinitionName, indent: str = "\t\t\t") -> List[str]:
+            rval = [f"{indent}{typ}" + (":" if typ in self.typeofs else "")]
             if typ in sorted(self.typeofs):
                 for tr in sorted(self.typeofs[typ]):
-                    rval += recurse_types(tr, indent + '\t')
+                    rval += recurse_types(tr, indent + "\t")
             return rval
 
-        rval = ['']
+        rval = [""]
         rval += [f"Classes: {len(self.schema.classes.keys())}"]
         rval += [summarize_refs(self.classrefs)]
 
@@ -277,8 +371,9 @@ class SchemaSynopsis:
         # Standalone ar classes that are both roots and leaves
         rval += [f"\tStandalone: {len(set(self.roots.classrefs).union(set(leaves)))}"]
         rval += [f"\tDeclared mixin: {len(self.mixins.classrefs)}"]
-        undeclared_mixins = set(self.mixinrefs.keys())\
-            .intersection(set(self.schema.classes.keys()) - set(self.mixins.classrefs))
+        undeclared_mixins = set(self.mixinrefs.keys()).intersection(
+            set(self.schema.classes.keys()) - set(self.mixins.classrefs)
+        )
         rval += [f"\tUndeclared mixin: {len(undeclared_mixins)}"]
         if undeclared_mixins:
             for udm in sorted(undeclared_mixins):
@@ -288,7 +383,7 @@ class SchemaSynopsis:
         if undefined_classes:
             rval += [f"\tUndefined references: {', '.join(undefined_classes)}"]
 
-        rval += ['']
+        rval += [""]
         rval += [f"Slots: {len(self.schema.slots.keys())}"]
         rval += [summarize_refs(self.slotrefs)]
         rval += [f"\tRoot: {len(self.roots.slotrefs)}"]
@@ -296,8 +391,9 @@ class SchemaSynopsis:
         rval += [f"\tLeaf: {len(leaves)}"]
         rval += [f"\tStandalone: {len(set(self.roots.classrefs).union(set(leaves)))}"]
         rval += [f"\tDeclared mixin: {len(self.mixins.slotrefs)}"]
-        undeclared_mixins = set(self.mixinrefs.keys()) \
-            .intersection(set(self.schema.slots.keys()) - set(self.mixins.slotrefs))
+        undeclared_mixins = set(self.mixinrefs.keys()).intersection(
+            set(self.schema.slots.keys()) - set(self.mixins.slotrefs)
+        )
         rval += [f"\tUndeclared mixin: {len(undeclared_mixins)}"]
         if undeclared_mixins:
             for udm in sorted(undeclared_mixins):
@@ -320,11 +416,12 @@ class SchemaSynopsis:
                     else:
                         unowned_slots.add(slotname)
         if n_unreferenced_descendants:
-            rval += [f"\tUnreferenced descendants of owned slots: {n_unreferenced_descendants}"]
+            rval += [
+                f"\tUnreferenced descendants of owned slots: {n_unreferenced_descendants}"
+            ]
         if unowned_slots:
             rval += [f"\t* Unowned slots: {', '.join(sorted(unowned_slots))}"]
 
-
         not_in_domain: Set[SlotDefinitionName] = set()
         domain_mismatches: Set[SlotDefinitionName] = set()
         unkdomains: Set[SlotDefinitionName] = set()
@@ -351,8 +448,10 @@ class SchemaSynopsis:
         if domain_mismatches:
             rval += [f"\t\tMismatches: {len(domain_mismatches)}"]
             for slotname in sorted(domain_mismatches):
-                rval.append(f'\t\t\tSlot: "{slotname}" declared domain: "{self.schema.slots[slotname].domain}" '
-                            f'actual domain(s): {", ".join(self.slotclasses[slotname])}')
+                rval.append(
+                    f'\t\t\tSlot: "{slotname}" declared domain: "{self.schema.slots[slotname].domain}" '
+                    f'actual domain(s): {", ".join(self.slotclasses[slotname])}'
+                )
         if unkdomains:
             rval += [f"\t* Unknown domain: {', '.join(sorted(unkdomains))}"]
         if emptydomains:
@@ -374,13 +473,15 @@ class SchemaSynopsis:
         if unknowns:
             rval += ["\t\tUnknown:"] + unknowns
 
-        shared_class_slots = set(self.schema.classes.keys()).intersection(set(self.schema.slots.keys()))
+        shared_class_slots = set(self.schema.classes.keys()).intersection(
+            set(self.schema.slots.keys())
+        )
         if shared_class_slots:
             rval += ["\nClasses and Slots with the same name:"]
             for ssc in sorted(shared_class_slots):
                 rval += [f"\t{ssc}"]
 
-        rval += ['']
+        rval += [""]
         rval += [f"Types: {len(self.schema.types)}"]
         rval += [summarize_refs(self.typerefs)]
         rval += ["\tBases:"]
@@ -390,5 +491,4 @@ class SchemaSynopsis:
                 if not self.schema.types[typ].typeof:
                     rval += recurse_types(typ)
 
-        return '\n'.join(rval)
-
+        return "\n".join(rval)
